#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['gpam_training']

package_data = \
{'': ['*']}

install_requires = \
['numpy==1.17.4', 'pandas==0.24.2', 'scikit-learn==0.22']

setup(name='gpam_training',
      version='0.0.2',
      description='Module to facilitate the integration of a sklearn training pipeline into a deploy and retraining system',
      author='João Robson',
      author_email=None,
      url=None,
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
     )
